package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.AuthenticationService;
import fr.itsonus.bousoleplussbackend.domain.auth.model.User;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Bundle;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Question;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories.BundleCacheProxyRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories.QuestionCacheProxyRepository;
import lombok.AllArgsConstructor;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@AllArgsConstructor
public class BundleUseCase {

    private final BundleCacheProxyRepository bundleCacheProxyRepository;
    private final AuthenticationService authenticationService;
    private final QuestionCacheProxyRepository questionCacheProxyRepository;

    public Bundle findById(Long id) {
        var user = authenticationService.getCurrentUser();
        return bundleCacheProxyRepository.findByIdAndUserId(id, user.getId());
    }

    public boolean existsForCurrentUser(Long id) {
        var user = authenticationService.getCurrentUser();
        try {
            return bundleCacheProxyRepository.findByIdAndUserId(id, user.getId()) != null;
        } catch (ResourceNotFoundException e) {
            return false;
        }
    }

    public List<Bundle> findAllForCurrentUser() {
        var user = authenticationService.getCurrentUser();
        return bundleCacheProxyRepository.findAllByUser(user);
    }

    public Bundle create(String label, List<Question> questions) {
        return this.create(label, questions, authenticationService.getCurrentUser());
    }

    public Bundle create(String label, List<Question> questions, User user) {
        return bundleCacheProxyRepository.save(label, questions, user);
    }

    public Iterable<Question> findQuestions(Long id, Long axeId) {
        return questionCacheProxyRepository.findQuestions(axeId, findById(id).getId());
    }
}
